"""This module introduces classes for handling different clusters."""
from abc import ABC, abstractmethod
from typing import Optional

from pykube import HTTPClient, KubeConfig


class Cluster(ABC):
    """Represents an abstract cluster."""

    _kube_client: Optional[HTTPClient]

    def __init__(self):
        self._kube_client = None

    @abstractmethod
    def create(self) -> HTTPClient:
        """Creates an instance of a cluster and returns HTTPClient to connect to it."""
        raise NotImplementedError

    @abstractmethod
    def destroy(self) -> None:
        """Destroys the cluster created earlier with a call to [create](Cluster.create)."""
        raise NotImplementedError

    @property
    def kube_client(self) -> Optional[HTTPClient]:
        """Returns the HTTP client you can use to access Kubernetes API of the cluster under test.

        Please refer to [pykube](https://pykube.readthedocs.io/en/latest/api/pykube.html) to get docs
        for [HTTPClient](https://pykube.readthedocs.io/en/latest/api/pykube.html#pykube.http.HTTPClient).
        """
        return self._kube_client


class ExistingCluster(Cluster):
    """Implementation of [Cluster](Cluster) that uses kube.config
    for an existing cluster.
    """

    kube_config_path: str

    def __init__(self, kube_config_path: str) -> None:
        super().__init__()
        self.kube_config_path = kube_config_path

    def create(self) -> HTTPClient:
        kube_config = KubeConfig.from_file(self.kube_config_path)
        self._kube_client = HTTPClient(kube_config)
        return self._kube_client

    def destroy(self) -> None:
        if self._kube_client is None:
            return
        self._kube_client.session.close()
        self._kube_client = None



from typing import Callable, List, NamedTuple, Optional

import yaml
from pykube import HTTPClient, ConfigMap

from .app_catalog import AppCatalogFactoryFunc
from .custom_resources import AppCR
from ..utils import YamlDict


class ConfiguredApp(NamedTuple):
    app: AppCR
    app_cm: Optional[ConfigMap]


AppFactoryFunc = Callable[[str, str, str, str, str, YamlDict], ConfiguredApp]


def app_factory_func(
    kube_client: HTTPClient, app_catalog_factory: AppCatalogFactoryFunc, created_apps: List[ConfiguredApp]
) -> AppFactoryFunc:
    def _app_factory(
        app_name: str,
        app_version: str,
        catalog_name: str,
        catalog_url: str,
        namespace: str = "default",
        config_values: YamlDict = None,
    ) -> ConfiguredApp:
        # TODO: include proper regexp validation
        if config_values is None:
            config_values = {}
        assert app_name != ""
        assert app_version != ""
        assert catalog_name != ""
        assert catalog_url != ""

        api_version = "application.giantswarm.io/v1alpha1"
        app_cm_name = "{}-testing-user-config".format(app_name)
        catalog = app_catalog_factory(catalog_name, catalog_url)
        kind = "App"

        app: YamlDict = {
            "apiVersion": api_version,
            "kind": kind,
            "metadata": {
                "name": app_name,
                "namespace": namespace,
                "labels": {"app": app_name, "app-operator.giantswarm.io/version": "1.0.0"},
            },
            "spec": {
                "catalog": catalog.metadata["name"],
                "version": app_version,
                "kubeConfig": {"inCluster": True},
                "name": app_name,
                "namespace": namespace,
            },
        }

        app_cm_obj: Optional[ConfigMap] = None
        if config_values:
            app["spec"]["config"] = {"configMap": {"name": app_cm_name, "namespace": namespace}}
            app_cm: YamlDict = {
                "apiVersion": "v1",
                "kind": "ConfigMap",
                "metadata": {"name": app_cm_name, "namespace": namespace},
                "data": {"values": yaml.dump(config_values)},
            }
            app_cm_obj = ConfigMap(kube_client, app_cm)
            app_cm_obj.create()

        app_obj = AppCR(kube_client, app)
        app_obj.create()
        created_apps.append(ConfiguredApp(app_obj, app_cm_obj))
        # TODO: wait until deployment is all ready
        # we return a new object here, so that user doesn't alter the one added to created_apps
        return ConfiguredApp(app_obj, app_cm_obj)

    return _app_factory



"""Different utilities required over the whole testing lib."""
import logging
import time
from typing import Dict, Any, List, TypeVar, Callable, Type

import pykube.exceptions
from pykube import HTTPClient, Job, Service, Deployment

# YamlValue = Union[int, float, str, bool, List['YamlValue'], 'YamlDict']
from requests import Response

YamlDict = Dict[str, Any]

logger = logging.getLogger(__name__)

# TODO: doesn't work as T for method below
T = TypeVar("T", bound=pykube.objects.NamespacedAPIObject)


def wait_for_namespaced_objects_condition(
    kube_client: HTTPClient,
    obj_type: Type[T],
    obj_names: List[str],
    objs_namespace: str,
    obj_condition_fun: Callable[[T], bool],
    timeout_sec: int,
    missing_ok: bool,
) -> List[T]:
    if len(obj_names) == 0:
        raise ValueError("'obj_names' list can't be empty.")

    retries = 0
    all_ready = False
    matching_objs: List[T] = []
    while retries < timeout_sec:
        response = obj_type.objects(kube_client).filter(namespace=objs_namespace)
        retries += 1
        matching_objs = []
        for name in obj_names:
            try:
                obj = response.get_by_name(name)
                assert obj is not None
                matching_objs.append(obj)
            except pykube.exceptions.ObjectDoesNotExist:
                if missing_ok:
                    pass
                else:
                    raise
        all_ready = len(matching_objs) == len(obj_names) and all(obj_condition_fun(obj) for obj in matching_objs)
        if all_ready:
            break
        time.sleep(1)

    if not all_ready:
        raise TimeoutError(f"Error waiting for object of type {obj_type} to match the condition.")

    return matching_objs


def _job_complete(job: Job) -> bool:
    complete = (
        "status" in job.obj
        and "conditions" in job.obj["status"]
        and len(job.obj["status"]["conditions"]) > 0
        and job.obj["status"]["conditions"][0]["type"] == "Complete"
    )
    return complete


def wait_for_jobs_to_complete(
    kube_client: HTTPClient, job_names: List[str], jobs_namespace: str, timeout_sec: int, missing_ok: bool = True
) -> List[Job]:
    result = wait_for_namespaced_objects_condition(
        kube_client, Job, job_names, jobs_namespace, _job_complete, timeout_sec, missing_ok
    )
    return result


def _deployment_running(d: Deployment) -> bool:
    complete = (
        d.ready and "availableReplicas" in d.obj["status"] and d.replicas == int(d.obj["status"]["availableReplicas"])
    )
    return complete


def wait_for_deployments_to_run(
    kube_client: HTTPClient,
    deployment_names: List[str],
    deployments_namespace: str,
    timeout_sec: int,
    missing_ok: bool = True,
) -> List[Deployment]:
    result = wait_for_namespaced_objects_condition(
        kube_client, Deployment, deployment_names, deployments_namespace, _deployment_running, timeout_sec, missing_ok,
    )
    return result


def proxy_http_request(client: HTTPClient, srv: Service, method, path, **kwargs) -> Response:
    """Template request to proxy of a Service.
    Args:
        :param client: HTTPClient to use.
        :param srv: Service you want to proxy.
        :param method: The http request method e.g. 'GET', 'POST' etc.
        :param path: The URI path for the request.
        :param kwargs: Keyword arguments for the proxy_http_get function.
    Returns:
        The Response data.
    """
    if "port" in kwargs:
        port = kwargs["port"]
    else:
        port = srv.obj["spec"]["ports"][0]["port"]
    kwargs["url"] = f"services/{srv.name}:{port}/proxy/{path}"
    kwargs["namespace"] = srv.namespace
    kwargs["version"] = srv.version
    return client.request(method, **kwargs)


def proxy_http_get(client: HTTPClient, srv: Service, path: str, **kwargs) -> Response:
    """Issue a GET request to proxy of a Service.
    Args:
        :param client: HTTPClient to use.
        :param srv: Service you want to proxy.
        :param path: The URI path for the request.
        :param kwargs: Keyword arguments for the proxy_http_get function.
    Returns:
        The response data.
    """
    return proxy_http_request(client, srv, "GET", path, **kwargs)


def proxy_http_post(client: HTTPClient, srv: Service, path: str, **kwargs) -> Response:
    """Issue a POST request to proxy of a Service.
    Args:
        :param client: HTTPClient to use.
        :param srv: Service you want to proxy.
        :param path: The URI path for the request.
        :param kwargs: Keyword arguments for the proxy_http_get function.
    Returns:
        The response data.
    """
    return proxy_http_request(client, srv, "POST", path, **kwargs)


def proxy_http_put(client: HTTPClient, srv: Service, path: str, **kwargs) -> Response:
    """Issue a PUT request to proxy of a Service.
    Args:
        :param client: HTTPClient to use.
        :param srv: Service you want to proxy.
        :param path: The URI path for the request.
        :param kwargs: Keyword arguments for the proxy_http_get function.
    Returns:
        The response data.
    """
    return proxy_http_request(client, srv, "PUT", path, **kwargs)


def proxy_http_delete(client: HTTPClient, srv: Service, path: str, **kwargs) -> Response:
    """Issue a DELETE request to proxy of a Service.
    Args:
        :param client: HTTPClient to use.
        :param srv: Service you want to proxy.
        :param path: The URI path for the request.
        :param kwargs: Keyword arguments for the proxy_http_get function.
    Returns:
        The response data.
    """
    return proxy_http_request(client, srv, "DELETE", path, **kwargs)



from typing import List, Iterable

import pytest

from .app import ConfiguredApp, AppFactoryFunc, app_factory_func
from .app_catalog import AppCatalogFactoryFunc, AppCatalogCR, app_catalog_factory_func
from .custom_resources import AppCR
from ..clusters import Cluster


@pytest.fixture(scope="module")
def app_catalog_factory(kube_cluster: Cluster) -> Iterable[AppCatalogFactoryFunc]:
    """Return a factory object, that can be used to configure new AppCatalog CRs
    for the 'app-operator' running in the cluster"""
    created_catalogs: List[AppCatalogCR] = []

    yield app_catalog_factory_func(kube_cluster.kube_client, created_catalogs)

    for catalog in created_catalogs:
        catalog.delete()
        # TODO: wait until finalizer is gone and object is deleted


@pytest.fixture(scope="module")
def app_factory(kube_cluster: Cluster, app_catalog_factory: AppCatalogFactoryFunc) -> Iterable[AppFactoryFunc]:
    """Returns a factory function which can be used to install an app using App CR"""

    created_apps: List[ConfiguredApp] = []

    yield app_factory_func(kube_cluster.kube_client, app_catalog_factory, created_apps)

    for created in created_apps:
        created.app.delete()
        if created.app_cm:
            created.app_cm.delete()
        # TODO: wait until finalizer is gone


@pytest.fixture(scope="module")
def kube_cluster_with_app_platform(
    kube_cluster: Cluster, app_catalog_factory: AppCatalogFactoryFunc
) -> Iterable[Cluster]:
    """Get a ready cluster based on '--cluster-type' command line argument. Additionally,
    preconfigure the cluster with Giant Swarm's Application Platform, including:
    - app-operator
    - chart-operator
    - chartmuseum (for storing custom build time charts)
    - AppCatalog Custom Resource configured for the chartmuseum."""
    # FIXME: implement
    # TODO:
    # - deploy app-operator
    # - deploy chartmuseum
    # - create new AppCatalog CR with app_catalog_factory to register chartmuseum as catalog
    raise NotImplementedError
    # yield kube_cluster
    # TODO:
    # - destroy app-operator
    # - destroy chartmuseum


@pytest.fixture(scope="module")
def my_chart() -> AppCR:
    """Returns AppCR that can be used to deploy the chart under test using the App Platform
    tools. The App resource is not yet deployed to the cluster. You need to call create()
    and delete() to manage its deployment"""
    # FIXME: implement
    raise NotImplementedError



from typing import Callable, List, Optional

from pykube import HTTPClient

from .custom_resources import AppCatalogCR

AppCatalogFactoryFunc = Callable[[str, Optional[str]], AppCatalogCR]


def __get_app_catalog_obj(catalog_name: str, catalog_uri: str, kube_client: HTTPClient) -> AppCatalogCR:
    app_catalog_cr = {
        "apiVersion": "application.giantswarm.io/v1alpha1",
        "kind": "AppCatalog",
        "metadata": {
            "labels": {"app-operator.giantswarm.io/version": "1.0.0", "application.giantswarm.io/catalog-type": ""},
            "name": catalog_name,
        },
        "spec": {
            "description": "Catalog for testing.",
            "storage": {"URL": catalog_uri, "type": "helm"},
            "title": catalog_name,
        },
    }
    return AppCatalogCR(kube_client, app_catalog_cr)


def app_catalog_factory_func(
    kube_client: HTTPClient, created_app_catalogs: List[AppCatalogCR]
) -> AppCatalogFactoryFunc:
    """Return a factory object, that can be used to configure new AppCatalog CRs
    for the 'app-operator' running in the cluster"""

    def _app_catalog_factory(name: str, url: Optional[str] = "") -> AppCatalogCR:
        if url == "":
            url = "https://giantswarm.github.io/{}-catalog/".format(name)
        for c in created_app_catalogs:
            if c.metadata["name"] == name:
                existing_url = c.obj["spec"]["storage"]["URL"]
                if existing_url == url:
                    return c
                raise ValueError(
                    "You requested creation of AppCatalog named {} with URL {} but it was already registered with URL "
                    "{}".format(name, url, existing_url)
                )

        app_catalog = __get_app_catalog_obj(name, str(url), kube_client)
        created_app_catalogs.append(app_catalog)
        app_catalog.create()
        # TODO: check that app catalog is present
        return app_catalog

    return _app_catalog_factory



import os
from pathlib import Path

from _pytest.config.argparsing import Parser

from .fixtures import chart_path  # noqa
from .fixtures import chart_version  # noqa
from .fixtures import chart_extra_info  # noqa
from .fixtures import values_file_path  # noqa
from .fixtures import kube_config  # noqa
from .fixtures import cluster_type  # noqa
from .fixtures import kube_cluster  # noqa
from .fixtures import _existing_cluster_factory  # noqa
from .fixtures import _kind_cluster_factory  # noqa
from .fixtures import _giantswarm_cluster_factory  # noqa

from .giantswarm_app_platform.fixtures import app_factory  # noqa
from .giantswarm_app_platform.fixtures import app_catalog_factory  # noqa

from .giantswarm_app_platform.apps.http_testing import stormforger_load_app_factory  # noqa
from .giantswarm_app_platform.apps.http_testing import gatling_app_factory  # noqa


def pytest_addoption(parser: Parser) -> None:
    group = parser.getgroup("helm-charts")
    group.addoption(
        "--cluster-type", action="store", default="existing", help="Select cluster type. Supported values: 'existing'."
    )
    group.addoption(
        "--kube-config",
        action="store",
        default=os.path.join(str(Path.home()), ".kube", "config"),
        help="The path to 'kube.config' file. Used when '--cluster-type existing' is used as well.",
    )
    group.addoption("--values-file", action="store", help="Path to the values file used for testing the chart.")
    group.addoption("--chart-path", action="store", help="The path to a helm chart under test.")
    group.addoption("--chart-version", action="store", help="Override chart version for the chart under test.")
    group.addoption(
        "--chart-extra-info",
        action="store",
        default="",
        help="Pass any additional info about the chart in the 'key1=val1,key2=val2' format",
    )



"""This module defines fixtures for testing Helm Charts."""
import logging
import sys
from typing import Callable, List, Iterable, Dict

import pytest
from _pytest.config import Config

from .clusters import ExistingCluster, Cluster

logger = logging.getLogger(__name__)


@pytest.fixture(scope="module")
def chart_path(pytestconfig: Config) -> str:
    """Return a path to the chart under test (from command line argument)."""
    return pytestconfig.getoption("chart_path")


@pytest.fixture(scope="module")
def chart_version(pytestconfig: Config) -> str:
    """Return a value that needs to be used as chart version override (from command line argument)."""
    return pytestconfig.getoption("chart_version")


def _parse_extra_info(info: str) -> Dict[str, str]:
    pairs = list(filter(None, info.split(",")))
    res_dict: Dict[str, str] = {}
    for pair in pairs:
        k, v = list(filter(None, pair.split("=")))
        res_dict[k] = v
    return res_dict


@pytest.fixture(scope="module")
def chart_extra_info(pytestconfig: Config) -> Dict[str, str]:
    """Return an optional dict of keywords and values passed to the test using '--chart-extra-info' config option."""
    arg = pytestconfig.getoption("chart_extra_info")
    return _parse_extra_info(arg)


@pytest.fixture(scope="module")
def values_file_path(pytestconfig: Config) -> str:
    """Return a path to the yaml file that needs to be used to configure chart under test
    (from command line argument).
    """
    return pytestconfig.getoption("values_file")


@pytest.fixture(scope="module")
def kube_config(pytestconfig: Config) -> str:
    """Return a path to the kube.config file that points to a running cluster with app
    catalog platform tools already installed. Used only if --cluster-type=existing (from command line argument)."""
    return pytestconfig.getoption("kube_config")


@pytest.fixture(scope="module")
def cluster_type(pytestconfig: Config) -> str:
    """Return a type of cluster to provide to the test environment. Currently supported values are:
    "existing"."""
    return pytestconfig.getoption("cluster_type")


ConfigFactoryFunction = Callable[[], Cluster]


@pytest.fixture(scope="module")
def _existing_cluster_factory(kube_config: str) -> ConfigFactoryFunction:
    def _fun() -> Cluster:
        return ExistingCluster(kube_config)

    return _fun


@pytest.fixture(scope="module")
def _kind_cluster_factory() -> ConfigFactoryFunction:
    def _fun() -> Cluster:
        # FIXME: implement
        raise NotImplementedError

    return _fun


@pytest.fixture(scope="module")
def _giantswarm_cluster_factory() -> ConfigFactoryFunction:
    def _fun() -> Cluster:
        # FIXME: implement
        raise NotImplementedError

    return _fun


@pytest.fixture(scope="module")
def kube_cluster(
    cluster_type: str,
    _existing_cluster_factory: ConfigFactoryFunction,
    _kind_cluster_factory: ConfigFactoryFunction,
    _giantswarm_cluster_factory: ConfigFactoryFunction,
) -> Iterable[Cluster]:
    """Return a ready Cluster object, which can already be used in test to connect
    to the cluster. Specific implementation used to provide the cluster depends
    on the '--cluster-type' command line option."""
    cluster: Cluster
    created_clusters: List[Cluster] = []
    if cluster_type == "existing":
        cluster = _existing_cluster_factory()
    elif cluster_type == "kind":
        cluster = _kind_cluster_factory()
    elif cluster_type == "giantswarm":
        cluster = _giantswarm_cluster_factory()
    else:
        raise ValueError("Unsupported cluster type '{}'.".format(cluster_type))

    logger.info("Creating new cluster of type '{}'.".format(cluster_type))
    cluster.create()
    logger.info("Cluster created")
    created_clusters.append(cluster)
    yield cluster

    for c in created_clusters:
        # noinspection PyBroadException
        try:
            logger.info("Destroying cluster")
            c.destroy()
            logger.info("Cluster destroyed")
        except Exception:
            exc = sys.exc_info()
            logger.error(
                "Error of type {} when destroying cluster. Value: {}\nStacktrace:\n{}".format(exc[0], exc[1], exc[2])
            )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pytest_helm_charts',
 'pytest_helm_charts.giantswarm_app_platform',
 'pytest_helm_charts.giantswarm_app_platform.apps']

package_data = \
{'': ['*']}

install_requires = \
['pykube-ng>=20.7.0,<21.0.0', 'pytest>=5.4.2,<6.0.0']

extras_require = \
{'docs': ['mkdocs>=1.1.2,<2.0.0', 'mkapi>=1.0.10,<2.0.0']}

entry_points = \
{'pytest11': ['helm-charts = pytest_helm_charts.plugin']}

setup_kwargs = {
    'name': 'pytest-helm-charts',
    'version': '0.1.8',
    'description': 'A plugin to provide different types and configs of Kubernetes clusters that can be used for testing.',
    'long_description': '[![build](https://github.com/giantswarm/pytest-helm-charts/workflows/build/badge.svg)](https://github.com/giantswarm/pytest-helm-charts/workflows/build/badge.svg)\n[![codecov](https://codecov.io/gh/giantswarm/pytest-helm-charts/branch/master/graph/badge.svg)](https://codecov.io/gh/giantswarm/pytest-helm-charts)\n[![Documentation Status](https://readthedocs.org/projects/pytest-helm-charts/badge/?version=latest)](https://pytest-helm-charts.readthedocs.io/en/latest/?badge=latest)\n[![PyPI Version](https://img.shields.io/pypi/v/pytest-helm-charts.svg)](https://pypi.org/project/pytest-helm-charts/)\n[![Python Versions](https://img.shields.io/pypi/pyversions/pytest-helm-charts.svg)](https://pypi.org/project/pytest-helm-charts/)\n[![Apache License](https://img.shields.io/badge/license-apache-blue.svg)](https://pypi.org/project/pytest-helm-charts/)\n\n# pytest-helm-charts\n\nA plugin to test helm charts on Kubernetes clusters.\n\nFull documentation (including API) available on <https://pytest-helm-charts.readthedocs.io/>.\n\n---\n\n## Features\n\nThis plugin provides a set of [pytest](https://docs.pytest.org/) fixtures that allow you to easily\nwrite tests for Helm charts and run them on Kubernetes clusters.\n\nIt can be also used to test Helm charts deployed using the Open Source\n[Giant Swarm App Platform](https://docs.giantswarm.io/basics/app-platform/).\n\nMost important features:\n\n- provides [pykube-ng](http://pykube.readthedocs.io/) interface to access Kubernetes clusters\n- provides [command line options](#usage) to configure the target cluster to run on\n- supports custom resource for the Giant Swarm App Platform:\n    - [App](https://docs.giantswarm.io/reference/cp-k8s-api/apps.application.giantswarm.io/)\n    - [AppCatalog](https://docs.giantswarm.io/reference/cp-k8s-api/appcatalogs.application.giantswarm.io/)\n- provides set of fixtures to easily work with Helm charts\n\n## Requirements\n\n- python 3.7+\n- pytest 5.4.2+\n- pykube-ng = 20.7.0+\n\n## Installation\n\nYou can install "pytest-helm-charts" via `pip` from `PyPI`:\n\n```\n$ pip install pytest-helm-charts\n```\n\n## Usage\n\n### Running your tests\n\nWhen you want to run your tests, you invoke `pytest` as usual, just passing additional\nflags on the command line. You can inspect them directly by running `pytest -h` and\nchecking the "helm-charts" section.\n\nCurrently, the only supported cluster type is `external`, which means the cluster is not\nmanaged by the test suite. You just point the test suite to a `kube.config` file,\nwhich can be used to connect to the Kubernetes API with `--kube-config` command line\noption. For creating development time clusters, we recommend using\n[KinD](https://kind.sigs.k8s.io/).\n\nIf you use this project to test Helm charts against Giant Swarm App Platform, the `existing`\ncluster must already have the platform components installed. Please refer to and use\nthe [`kube-app-testing`](https://github.com/giantswarm/kube-app-testing) tool to easily\ncreate KinD based clusters with all the components already installed.\n\n### Writing tests\n\nThe easiest way to get started is by checking our\n[examples](https://pytest-helm-charts.readthedocs.io/en/latest/examples/basic).\n\nThe list of available fixtures can be found by running `pytest --fixtures`, but\nyou can also just check [the most important fixtures](fixtures.md).\n\nFull [API docs](https://pytest-helm-charts.readthedocs.io/en/latest/api/pytest_helm_charts/)\nare also available.\n\n## Contributing\n\nContributions are very welcome.\n\n- Project is managed with [`poetry`](https://python-poetry.org/),\n  to start developing run `poetry install`\n- Tests for all supported python versions can be run with [`tox`](https://tox.readthedocs.io/):\n  `poetry run tox -- --log-cli-level info tests/`\n- Please ensure\n  [the coverage](https://codecov.io/gh/giantswarm/pytest-helm-charts/)\n  at least stays the same before you submit a pull request.\n\n## License\n\nSee [LICENSE](LICENSE).\n\n## Changelog\n\nSee [CHANGELOG.md](CHANGELOG.md).\n\n## Issues\n\nIf you encounter any problems, please [file an issue](https://github.com/giantswarm/pytest-helm-charts/issues) along with a detailed description.\n',
    'author': 'Łukasz Piątkowski',
    'author_email': 'lukasz@giantswarm.io',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/giantswarm/pytest-helm-chart',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from pykube.objects import APIObject, NamespacedAPIObject


class AppCatalogCR(APIObject):
    version = "application.giantswarm.io/v1alpha1"
    endpoint = "appcatalogs"
    kind = "AppCatalog"


class AppCR(NamespacedAPIObject):
    version = "application.giantswarm.io/v1alpha1"
    endpoint = "apps"
    kind = "App"
